
* MONTE CARLO TO INVESTIGATE THE PERFORMANCE OF TREATMENT TESTS
* WITHIN VARIOUS MODELS FOR MULTI-LEVEL DATA




clear

set seed 7654321

* set obs to total number of observations (nT), and number of tasks T.
* if necessary, set number of clusters, J
* chosen obs also needs to go in simulate command at end of do-file

set obs 2000
scalar nT=_N

scalar T=50
scalar n=nT/T

* chosen T must appear in second option of next two commands

egen i=seq(), f(1) b(50)
egen t=seq(), f(1) t(50)
egen j=seq(), f(1) b(200)
xtset i t

* single simulation

* set treatment effect:

scalar treatment_effect=0.50

* between subject treatment:

gen d=i>n/2

* within subject treatment:

*gen d=t>T/2


* single simulation

scalar delta=treatment_effect

gen double x=10*uniform()

gen  e=rnormal()

bysort i: generate u=rnormal() if _n==1
bysort i: replace u=u[1] if u==.

bysort j: generate v=rnormal() if _n==1
bysort j: replace v=v[1] if v==.

gen y=0.5+delta*d+0.1*x+u+v+e

*MODEL 1: OLS no clustering

regress y d x
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt1=b1/sqrt(v1)
scalar pp1=1-normal(tt1)
scalar list tt1 pp1

* MODEL 2: OLS clustering at subject level

regress y d x, vce(cluster i)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt2=b1/sqrt(v1)
scalar pp2=1-normal(tt2)
scalar list tt2 pp2

*MODEL 3: OLS clustering at session level

regress y d x, vce(cluster j)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt3=b1/sqrt(v1)
scalar pp3=1-normal(tt3)
scalar list tt3 pp3

* MODEL 4: random effects, no clustering

xtreg y d x
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt4=b1/sqrt(v1)
scalar pp4=1-normal(tt4)
scalar list tt4 pp4

* MODEL 5: random effects, clustering at subject level

xtreg y d x,  vce(cluster i)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt5=b1/sqrt(v1)
scalar pp5=1-normal(tt5)
scalar list tt5 pp5

* MODEL 6: random effects, clustering at session level

xtreg y d x,  vce(cluster j)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt6=b1/sqrt(v1)
scalar pp6=1-normal(tt6)
scalar list tt6 pp6

* MODEL 7: multi-level model (subject random effect and session random effect)

xtmixed y d x   || j: || i: 
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt7=b1/sqrt(v1)
scalar pp7=1-normal(tt7)
scalar list tt7 pp7


* NOW MONTE CARLO

capt prog drop multi_level_monte
   
   program define multi_level_monte, rclass
        syntax [, obs(integer 1)  ]
        set obs `obs'
		keep x i t j d
		tempvar e u v y 
		
gen  e=rnormal()

bysort i: generate u=rnormal() if _n==1
bysort i: replace u=u[1] if u==.

bysort j: generate v=rnormal() if _n==1
bysort j: replace v=v[1] if v==.

gen y=0.5+delta*d+0.1*x+u+v+e

regress y d x
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt1=b1/sqrt(v1)
scalar pp1=1-normal(tt1)

regress y d x, vce(cluster i)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt2=b1/sqrt(v1)
scalar pp2=1-normal(tt2)


regress y d x, vce(cluster j)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt3=b1/sqrt(v1)
scalar pp3=1-normal(tt3)
scalar list tt3 pp3


xtreg y d x
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt4=b1/sqrt(v1)
scalar pp4=1-normal(tt4)


xtreg y d x,  vce(cluster i)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt5=b1/sqrt(v1)
scalar pp5=1-normal(tt5)


xtreg y d x,  vce(cluster j)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt6=b1/sqrt(v1)
scalar pp6=1-normal(tt6)


xtmixed y d x   || j: || i: 
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt7=b1/sqrt(v1)
scalar pp7=1-normal(tt7)

		
return scalar pp1 = pp1
return scalar pp2 = pp2
return scalar pp3 = pp3
return scalar pp4 = pp4
return scalar pp5 = pp5
return scalar pp6 = pp6
return scalar pp7 = pp7

end

* FIND SIZE OF EACH TEST (BETWEEN-SUBJECT)

* CREATE TREATMENT DUMMY FOR BETWEEN-SUBJECT TREATMENT:

replace d=i>n/2
*replace d=t>T/2

* SET TREATMENT EFFECT (TO ZERO OR TO treatment_effect)

scalar delta=0.5

simulate pp1=r(pp1) pp2=r(pp2) pp3=r(pp3) pp4=r(pp4) pp5=r(pp5) pp6=r(pp6) pp7=r(pp7), reps(100): multi_level_monte, obs(2000)

gen reject1=pp1<0.05
gen reject2=pp2<0.05
gen reject3=pp3<0.05
gen reject4=pp4<0.05
gen reject5=pp5<0.05
gen reject6=pp6<0.05
gen reject7=pp7<0.05

* REPORT SIZE OF EACH TEST (BETWEEN-SUBJECT):

summ reject*

