
** ONLY MULTI-LEVEL MODEL
** VARY n and T

clear

*set seed 654321

* set number of subjects n, number of tasks T, and number of groups J
* make sure n is divisible by J


local n=40
local T=100
local J=20

local nT=`n'*`T'
local obspg=`nT'/`J'


set obs `nT'


egen i=seq(), f(1) b(`T')
egen t=seq(), f(1) t(`T')
egen j=seq(), f(1) b(`obspg')
tsset i t

* between subject treatment:

*gen d=`i'>n/2

* within subject treatment:

gen d=t>`T'/2

* set treatment effect:

scalar delta=0.05

* single simulation

gen double x=10*uniform()

gen  e=rnormal()

bysort i: generate u=rnormal() if _n==1
bysort i: replace u=u[1] if u==.

bysort j: generate v=rnormal() if _n==1
bysort j: replace v=v[1] if v==.

gen y=0.5+delta*d+0.1*x+u+v+e

* multi-level model (subject random effect and session random effect)

xtmixed y d x   || j: || i: 
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt=b1/sqrt(v1)
scalar pp=1-normal(tt)
scalar list tt pp


* NOW MONTE CARLO

capt prog drop multi_level_monte
   
   program define multi_level_monte, rclass
        syntax [, obs(integer 1)  ]
        set obs `obs'
		keep x i j t d
		tempvar e u v y 
		

gen e=rnormal()

bysort i: generate u=rnormal() if _n==1
bysort i: replace u=u[1] if u==.

bysort j: generate v=rnormal() if _n==1
bysort j: replace v=v[1] if v==.

gen y=0.5+delta*d+0.1*x+u+v+e


xtmixed y d x   || j: || i: , emiter(20)
mat b=e(b)
scalar b1=b[1,1]
mat v=e(V)
scalar v1=v[1,1]
scalar tt=b1/sqrt(v1)
scalar pp=1-normal(tt)

return scalar pp = pp

end




simulate pp=r(pp), reps(100): multi_level_monte, obs(`nT')


gen reject=pp<0.05

summ reject


