
* PART 5: FINITE MIXTURE MODELS

*  MIXTURE OF TWO NORMALS

program drop _all
set more off

*  LIKELIHOOD EVALUATION PROGRAM STARTS HERE:

program define mixture
	args logl mu1 sig1 mu2 sig2 p
	tempvar f1 f2

* GENERATE TYPE-CONDITIONAL DENSITIES:	
	
	quietly gen double `f1'=(1/`sig1')*normalden((y-`mu1')/`sig1')
	quietly gen double `f2'=(1/`sig2')*normalden((y-`mu2')/`sig2')
	
* COMBINE TYPE-CONDITIONAL DENSITIES WITH MIXING PROPORTIONS 
* TO GENERATE MARGINAL DENSITY, AND TAKE LOG.  THIS IS THE FUNCTION THAT
* NEEDS TO BE MAXIMISED WHEN SUMMED OVER THE SAMPLE:
		
	quietly replace `logl'=ln(`p'*`f1'+(1-`p')*`f2')

* GENERATE THE POSTERIOR TYPE PROBABILITIES, AND MAKE THEM
* AVAILABLE OUTSIDE THE PROGRAM:
	
	quietly replace postp1=`p'*`f1'/(`p'*`f1'+(1-`p')*`f2')
	quietly replace postp2=(1-`p')*`f2'/(`p'*`f1'+(1-`p')*`f2')

	quietly putmata postp1, replace
	quietly putmata postp2, replace
	
	end

* END OF LIKELIHOOD EVALUATION PROGRAM	
	
* READ DATA:
	
use mixture_sim, clear	

* INITIALISE TWO POSTERIOR PROBABILITY VARIABLES:
	
gen postp1=.
gen postp2=.

* SPECIFY STARTING VALUES, AND APPLY ML:

mat start=(3, 1.5, 6, 1.5, .5)
ml model lf mixture /mu1 /sig1 /mu2 /sig2 /p
ml init start, copy
ml maximize

* EXTRACT POSTERIOR TYPE PROBABILITY, AND PLOT THEM AGAINST y:

drop postp1 postp2
getmata postp1
getmata postp2

sort y
line  postp1 postp2 y , lpattern(l -)

*  FMM command

fmm 2: regress y


*   LEVEL-K MODEL FOR THE BEAUTY CONTEST GAME

clear
program drop _all
set more off

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE:

program define beauty_mixture
	args logl p1 p2 p3 p4 p5 sig
	tempvar  f0 f1 f2 f3 f4 f5 l

	quietly{
	
	gen double `f0'=0.01
	gen double `f1'=(1/`sig')*normalden((y-33.5)/`sig')
	gen double `f2'=(1/`sig')*normalden((y-22.4)/`sig')
	gen double `f3'=(1/`sig')*normalden((y-15.0)/`sig')
	gen double `f4'=(1/`sig')*normalden((y-10.1)/`sig')
	gen double `f5'=(1/`sig')*normalden((y-0)/`sig')
	
	gen double `l'=(1-`p1'-`p2'-`p3'-`p4'-`p5')*`f0'   ///
	+`p1'*`f1'+`p2'*`f2'+`p3'*`f3'+`p4'*`f4'+`p5'*`f5'
	
	replace postp0=(1-`p1'-`p2'-`p3'-`p4'-`p5')*`f0'/`l'
	replace postp1=`p1'*`f1'/`l'
	replace postp2=`p2'*`f2'/`l'
	replace postp3=`p3'*`f3'/`l'
	replace postp4=`p4'*`f4'/`l'
	replace postp5=`p5'*`f5'/`l'
	
	replace `logl'=ln(`l')
		
 putmata postp0, replace
 putmata postp1, replace
 putmata postp2, replace
 putmata postp3, replace
 putmata postp4, replace
 putmata postp5, replace
	
	}
	
	end

* END OF LIKELIHOOD EVALUATION PROGRAM

* READ DATA	
	
use beauty_sim, clear	

hist y, disc xtitle("guess")

* INITIALISE POSTERIOR TYPE PROBABILITIES
	
gen postp0=.
gen postp1=.
gen postp2=.
gen postp3=.
gen postp4=.
gen postp5=.

* SET STARTING VALUES AND RUN ML

mat start=(0.3, 0.4, 0.1, 0.1,0.05, 2)
ml model lf beauty_mixture /p1 /p2 /p3 /p4 /p5 /sig
ml init start, copy
ml maximize

* DEDUCE PROBABILITY OF LEVEL-0 TYPE

nlcom p0: 1-_b[/p1]-_b[/p2]-_b[/p3]-_b[/p4]-_b[/p5]

* EXTRACT POSTERIOR TYPE PROBABILITIES

drop postp*
getmata postp0
getmata postp1
getmata postp2
getmata postp3
getmata postp4
getmata postp5

* PLOT TYPE PROBABILITIES AGAINST GUESS

sort y

line  postp0 postp1 postp2 postp3 postp4 postp5 y , lpattern(- l l l l l) ///
 xline(21.7) xtitle("guess") ytitle("posterior type probability") legend(off)



*  A MIXTURE MODEL FOR GIVING IN A PUBLIC GOODS EXPERIMENT

prog drop _all
set more off

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE:

program define pg_mixture
	args todo b logl
	tempvar p1_1 p2_1 p3_1 p1_2 p2_2 p3_2 p1_3 p2_3 p3_3 p1 p2 p3 pp1 pp2 pp3 pp w
		
	tempname xb1 xb2 sig1 sig2 w0 w1 p_rec p_str

* ASSIGN PARAMETER NAMES TO THE ELEMENTS OF THE PARAMETER VECTOR b:
	
	mleval `xb1' = `b', eq(1) 
	mleval `xb2' = `b', eq(2) 
	mleval `sig1' = `b', eq(3) scalar
	mleval `sig2'=`b', eq(4) scalar
	mleval `w0'=`b', eq(5) scalar
	mleval `w1'=`b', eq(6) scalar
	mleval `p_rec'=`b', eq(7) scalar
	mleval `p_str'=`b', eq(8) scalar
	
	quietly{
	
* INITIALISE THE p* VARIABLES WITH MISSING VALUES:
	
	gen double `p1_1'=.
	gen double `p2_1'=.
	gen double `p3_1'=.
	gen double `p1_2'=.
	gen double `p2_2'=.
	gen double `p3_2'=.
	gen double `p1_3'=.
	gen double `p2_3'=.
	gen double `p3_3'=.
	
	gen double `p1'=.
	gen double `p2'=.
	gen double `p3'=.
	
	gen double `pp1'=.
	gen double `pp2'=.
	gen double `pp3'=.
	gen double `pp'=.
	
* GENERATE THE TREMBLE PROBABILITY:
	
	gen double `w'=`w0'*exp(`w1'*tsk_1)

* COMPUTE TYPE-CONDITIONAL DENSITIES UNDER REGIME 1:
			
		replace `p1_1'=(1-`w')*normal(-`xb1'/`sig1')+`w'/11
		replace `p2_1'=(1-`w')*normal(-`xb2'/`sig2')+`w'/11
		replace `p3_1'=1-(10/11)*`w'

* COMPUTE TYPE-CONDITIONAL DENSITIES UNDER REGIME 2:
		
		replace `p1_2'=(1-`w')*(1/`sig1')*normalden((y-`xb1')/`sig1')+`w'/11
		replace `p2_2'=(1-`w')*(1/`sig2')*normalden((y-`xb2')/`sig2')+`w'/11
		replace `p3_2'=`w'/11

* COMPUTE TYPE-CONDITIONAL DENSITIES UNDER REGIME 3:
		
		replace `p1_3'=(1-`w')*(1-normal((10-`xb1')/`sig1'))+`w'/11
		replace `p2_3'=(1-`w')*(1-normal((10-`xb2')/`sig2'))+`w'/11
		replace `p3_3'=`w'/11

* MATCH TYPE-CONDITIONAL DENSITIES TO ACTUAL REGIMES (d IS REGIME):		
		
		replace `p1' = (d==1)*`p1_1'+(d==2)*`p1_2'+(d==3)*`p1_3'
		replace `p2' = (d==1)*`p2_1'+(d==2)*`p2_2'+(d==3)*`p2_3'
		replace `p3' = (d==1)*`p3_1'+(d==2)*`p3_2'+(d==3)*`p3_3'

* FIND PRODUCT OF TYPE-CONDITIONAL DENSITIES FOR EACH SUBJECT:		

		by i: replace `pp1'=exp(sum(ln(max(`p1',1e-12))))
		by i: replace `pp2'=exp(sum(ln(max(`p2',1e-12))))
		by i: replace `pp3'=exp(sum(ln(max(`p3',1e-12))))

* COMBINE TYPE-CONDITIONAL DENSITIES TO OBTAIN MARGINAL DENSITY FOR EACH SUBJECT
* (ONLY REQUIRED IN FINAL ROW FOR EACH SUBJECT):
		
		replace `pp'=`p_rec'*`pp1'+`p_str'*`pp2'+(1-`p_rec'-`p_str')*`pp3'
		replace `pp'=. if last~=1

* SPECIFY (LOG-LIKELIHOOD) FUNCTION WHOSE SUM OVER SUBJECTS IS TO BE MAXIMISED
		
		mlsum `logl'=ln(`pp') if last==1

* GENERATE POSTERIOR TYPE PROBABILITIES, AND MAKE THESE AVAILABLE OUTSIDE THE PROGRAM
		
		replace postp1=`p_rec'*`pp1'/`pp'
		replace postp2=`p_str'*`pp2'/`pp'
		replace postp3=(1-`p_rec'-`p_str')*`pp3'/`pp'
		
		putmata postp1, replace
		putmata postp2, replace
		putmata postp3, replace
	
	}
					
	end

* END OF LOG-LIKELIHOOD EVALUATION PROGRAM
	
clear
set more off

* READ DATA

use "bardsley", clear

* PLOT CONTRIBUTIONS AGAINST TASK BY SUBJECT

tsset i t
xtline y

by i: gen last=_n==_N

gen int d=1
replace d=2 if y>0
replace d=3 if y==10

gen double ord_1=ord-1
gen double tsk_1=tsk-1

* SET MEDIAN OF PREVIOUS CONTRIBUTIONS TO 8 FOR SUBJECTS IN FIRST POSITION:

replace med=8 if ord==1

*  SPECIFY EXPLANATORY-VARIABLE LISTS FOR RECIPROCATOR (LIST1) 
*  AND STRATEGIST (LIST2) EQUATIONS:

local list1 "med tsk_1"
local list2 "ord_1 tsk_1"

* INITIALISE VARIABLES TO BE USED FOR POSTERIOR TYPE PROBABILITIES:

gen postp1=.
gen postp2=.
gen postp3=.

* SPECIFY STARTING VALUES:

mat start=(0.57,-0.10,6.1,-0.93,-0.05,5.2,3.3,3.7,0.11,-0.05,0.26,0.49)

* SPECIFY LIKELIHOOD EVALUATOR, PROGRAM, AND PARAMETER NAMES:

ml model d0 pg_mixture (=`list1') (=`list2') /sig1 /sig2 /w0  /w1  /p_rec /p_str
ml init start, copy

* USE ML COMMAND TO MAXIMISE LOG-LIKELIHOOD, AND STORE RESULTS AS "WITH_TREMBLE":

ml max, trace search(norescale)
est store with_tremble

* COMPUTE THIRD MIXING PROPORTION USING DELTA METHOD:

nlcom p_fr: 1-_b[/p_rec]-_b[/p_str]

* EXTRACT POSTERIOR TYPE PROBABILITIES AND PLOT THEM AGAINST 
*  NUMBER OF ZERO CONTRIBUTIONS:

drop postp1 postp2 postp3

getmata postp1
getmata postp2
getmata postp3

label variable postp1 "rec"
label variable postp2 "str"
label variable postp3 "fr"

by i: gen n_zero=sum(y==0)

scatter postp1 postp2 postp3 n_zero if last==1, title("with tremble") ///
ytitle("posterior probability") msymbol(x Dh Sh) jitter(3) saving(with, replace)

* ESTIMATE MODEL WITHOUT TREMBLE, AND STORE RESULTS AS "WITHOUT_TREMBLE":

constraint 1 _b[/w0]=0.00
constraint 2 _b[/w1]=0.00

ml model d0 pg_mixture (=`list1') (=`list2') ///
/sig1 /sig2 /w0  /w1  /p_rec /p_str, constraints(1 2)

ml init start, copy
ml max, trace search(norescale)
est store without_tremble

nlcom p_fr: 1-_b[/p_rec]-_b[/p_str]

* EXTRACT AND PLOT POSTERIOR TYPE PROBABILITIES FOR MODEL WITHOUT TREMBLE:

drop postp1 postp2 postp3

getmata postp1
getmata postp2
getmata postp3

label variable postp1 "rec"
label variable postp2 "str"
label variable postp3 "fr"

scatter postp1 postp2 postp3 n_zero if last==1, title("without tremble") ///
ytitle("posterior probability") msymbol(x Dh Sh) jitter(3) saving(without, replace)

* CARRY OUT LIKELIHOOD RATIO TEST FOR PRESENCE OF TREMBLE:

lrtest with_tremble without_tremble

* COMBINE THE TWO POSTERIOR PROBABILITY PLOTS

gr combine with.gph without.gph

erase with.gph
erase without.gph


